// Variables for UBlox-7M setup
//boolean gpsStatus[] = {false, false, false, false, false, false, false, false, false, false};
  //Settings Array contains the following settings: [0]NavMode, [1]DataRate1, [2]DataRate2, [3]PortRateByte1, [4]PortRateByte2, [5]PortRateByte3,
  //[6]NMEA GLL Sentence, [7]NMEA GSA Sentence, [8]NMEA GSV Sentence, [9]NMEA RMC Sentence, [10]NMEA VTG Sentence  {11} Powermode [12] SBAS mode
  //NavMode: [0]
  //Stationary Mode    = 0x02
  //Pedestrian Mode    = 0x03
  //Automotive Mode    = 0x04
  //Sea Mode           = 0x05
  //Airborne < 1G Mode = 0x06
  //
  //DataRate: [1] [2]
  //1Hz     = 0xE8 0x03 // once every second is optimal
  //2Hz     = 0xF4 0x01 // do not use !!
  //3.33Hz  = 0x2C 0x01 // do not use !!
  //4Hz     = 0xFA 0x00 // do not use !!
  //
  //PortRate: [3] [4] [5]
  //4800   = C0 12 00
  //9600   = 80 25 00
  //19200  = 00 4B 00  **SOFTWARESERIAL LIMIT FOR ARDUINO UNO R3!**
  //38400  = 00 96 00  **SOFTWARESERIAL LIMIT FOR ARDUINO MEGA 2560!**
  //57600  = 00 E1 00
  //115200 = 00 C2 01  ** preferred for GPSdo hardware serial
  //230400 = 00 84 03
  //
  //NMEA Messages: [6] [7] [8] [9] [10]
  //OFF = 0x00
  //ON  = 0x01
  //
  //RXM power managenent: [11]
  //0x00 = Continous mode
  //0x01 = Power save mode
  //
  // SBAS mode [12]
  //0x00 = SBAS disabled
  //0x01 = SBAS enabled
  //
// byte settingsArray[] = {0x02, 0xE8, 0x03, 0x80, 0x25, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00}; // 9600 baud
byte settingsArray[] = {0x02, 0xE8, 0x03, 0x00, 0x4B, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00}; // 115200 baud
int Gps_Baud_Rate;
bool GpsStatus = true;
unsigned long SafeWait = 250;

void UbloxSmartWait ( unsigned long DelayTime)
{
  unsigned long TmpTim = millis();
  while (millis()-TmpTim <= DelayTime)
  {
    if(Serial1.available())
    {
      char c = Serial1.read();  
    }
  }
}

void printHex(uint8_t *data, uint8_t length) // prints 8-bit data in hex
{
  char tmp[length*2+1];
  byte first ;
  int j=0;
  for (byte i = 0; i < length; i++)
  {
    first = (data[i] >> 4) | 48;
    if (first > 57) tmp[j] = first + (byte)7;
    else tmp[j] = first ;
    j++;

    first = (data[i] & 0x0F) | 48;
    if (first > 57) tmp[j] = first + (byte)7;
    else tmp[j] = first;
    j++;
  }
  tmp[length*2] = 0;
  for (byte i = 0, j = 0; i < sizeof(tmp); i++) {
    if (j == 1) {
      j = 0;
    }
    else j++;
  }
}

byte getUBX_ACK(byte *msgID) {
  byte CK_A = 0, CK_B = 0;
  byte incoming_char;
  boolean headerReceived = false;
  unsigned long ackWait = millis();
  byte ackPacket[10] = {0xB5, 0x62, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  int i = 0;
  while (1) {
    if (Serial1.available()) {
      incoming_char = Serial1.read();
      if (incoming_char == ackPacket[i]) {
        i++;
      }
      else if (i > 2) {
        ackPacket[i] = incoming_char;
        i++;
      }
    }
    if (i > 9) break;
    if ((millis() - ackWait) > 1500) {
        UbloxSmartWait (SafeWait);
        return 5;
    }
    if (i == 4 && ackPacket[3] == 0x00) {
      return 1;
    }
  }

  for (i = 2; i < 8 ;i++) {
  CK_A = CK_A + ackPacket[i];
  CK_B = CK_B + CK_A;
  }
  if (msgID[0] == ackPacket[6] && msgID[1] == ackPacket[7] && CK_A == ackPacket[8] && CK_B == ackPacket[9]) {
    return 10;
        }
  else {
    UbloxSmartWait (SafeWait);
    return 1;
  }
}

void setBaud(byte baudSetting) {
  if (baudSetting == 0x12) Serial1.begin(4800);
  if (baudSetting == 0x25) Serial1.begin(9600);
  if (baudSetting == 0x4B) Serial1.begin(19200);
  if (baudSetting == 0x96) Serial1.begin(38400);
  if (baudSetting == 0xE1) Serial1.begin(57600);
  if (baudSetting == 0xC2) Serial1.begin(115200);
  if (baudSetting == 0x84) Serial1.begin(230400);
}

void sendUBX(byte *UBXmsg, byte msgLength) {
  for(int i = 0; i < msgLength; i++) {
    Serial1.write(UBXmsg[i]);
    Serial1.flush();
  }
  Serial1.println();
  Serial1.flush();
}

void calcChecksum(byte *checksumPayload, byte payloadSize) {
  byte CK_A = 0, CK_B = 0;
  for (int i = 0; i < payloadSize ;i++) {
    CK_A = CK_A + *checksumPayload;
    CK_B = CK_B + CK_A;
    checksumPayload++;
  }
  *checksumPayload = CK_A;
  checksumPayload++;
  *checksumPayload = CK_B;
}

int Find_Gps_Baud (byte *settingsArrayPointer) {
  // Depending on how long the GPS module has been without power.
  // After long period the baud rate is default to 9600
  // After a short period the working baud rate should work.
  // For the setup we make sure the GPS is at 9600.
  // First we try 9600
  byte gpsSetSuccess = 0;
  setBaud(0x25);   // select 9600 baudrate
  UbloxSmartWait (SafeWait);
  while(gpsSetSuccess < 3) {
    byte lowerPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x80, 0x25, 0x00, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    calcChecksum(&lowerPortRate[2], sizeof(lowerPortRate) - 4);
    sendUBX(lowerPortRate, sizeof(lowerPortRate));
    gpsSetSuccess += getUBX_ACK(&lowerPortRate[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;  //timeout and retry
  }
  if (gpsSetSuccess == 10) {
    return 9600; 
  }
  // Now try the configured gps baud rate and set back to 9600
  setBaud(settingsArrayPointer[4]);
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    //Generate the configuration string for Baud Rate
    byte setPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, settingsArrayPointer[3], settingsArrayPointer[4], settingsArrayPointer[5], 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    calcChecksum(&setPortRate[2], sizeof(setPortRate) - 4);
    sendUBX(setPortRate, sizeof(setPortRate));
    gpsSetSuccess += getUBX_ACK(&setPortRate[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;  //timeout and retry
  }
  if (gpsSetSuccess == 10) {
    UbloxSmartWait (SafeWait);
    gpsSetSuccess = 0;
    while(gpsSetSuccess < 3) {
      byte lowerPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x80, 0x25, 0x00, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
      calcChecksum(&lowerPortRate[2], sizeof(lowerPortRate) - 4);
      sendUBX(lowerPortRate, sizeof(lowerPortRate));
      setBaud(0x25);   // select 9600 baudrate
      gpsSetSuccess += getUBX_ACK(&lowerPortRate[2]); //Passes Class ID and Message ID to the ACK Receive function
      if (gpsSetSuccess == 5) gpsSetSuccess -= 4;  //timeout and retry
    }
    if (gpsSetSuccess == 10) {
      UbloxSmartWait (SafeWait);
      return 9600; 
    }
  }
  // Everything failed. Throw hardware error 
  return 0; 
}

void configureUblox(byte *settingsArrayPointer) {
  byte gpsSetSuccess = 0;
  char GPAsuccess[] = "         ";
  //Configuring u-Blox GPS initial state...
  //Generate the configuration string for Navigation Mode NAV5
  byte setNav[] = {0xB5, 0x62, 0x06, 0x24, 0x24, 0x00, 0xFF, 0xFF, *settingsArrayPointer, 0x03, 0x00, 0x00, 0x00, 0x00, 0x10, 0x27, 0x00, 0x00, 0x05, 0x00, 0xFA, 0x00, 0xFA, 0x00, 0x64, 0x00, 0x2C, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setNav[2], sizeof(setNav) - 4);

  //Generate the configuration string for Data Rate 
  byte setDataRate[] = {0xB5, 0x62, 0x06, 0x08, 0x06, 0x00, settingsArrayPointer[1], settingsArrayPointer[2], 0x01, 0x00, 0x01, 0x00, 0x00, 0x00};
  calcChecksum(&setDataRate[2], sizeof(setDataRate) - 4);

  //Generate the configuration string for Baud Rate
  byte setPortRate[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00, 0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, settingsArrayPointer[3], settingsArrayPointer[4], settingsArrayPointer[5], 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setPortRate[2], sizeof(setPortRate) - 4);

  // GLL messages on or off
  byte setGLL[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x01, settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], settingsArrayPointer[6], 0x01, 0x00, 0x00};
  calcChecksum(&setGLL[2], sizeof(setGLL) - 4);
  
  // GSA messages on or off
  byte setGSA[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x02, settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], settingsArrayPointer[7], 0x01, 0x00, 0x00};
  calcChecksum(&setGSA[2], sizeof(setGSA) - 4);
  
  // GSV messages on or off
  byte setGSV[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x03, settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], settingsArrayPointer[8], 0x01, 0x00, 0x00};
  calcChecksum(&setGSV[2], sizeof(setGSV) - 4);
  
  // RMC messages on or off
  byte setRMC[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x04, settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], settingsArrayPointer[9], 0x01, 0x00, 0x00};
  calcChecksum(&setRMC[2], sizeof(setRMC) - 4);
  
  // VTG messages on or off
  byte setVTG[] = {0xB5, 0x62, 0x06, 0x01, 0x08, 0x00, 0xF0, 0x05, settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], settingsArrayPointer[10], 0x00, 0x00, 0x00};
  calcChecksum(&setVTG[2], sizeof(setVTG) - 4);
  
  // Power management
  byte setPWRMG[] = {0xB5, 0x62, 0x06, 0x11, 0x02, 0x00, 0x08, settingsArrayPointer[11], 0x00, 0x00};
  calcChecksum(&setPWRMG[2], sizeof(setPWRMG) - 4);

  // Timepulse TP5 1Hz = geen sat 10MHz is fix
  byte setTP5[] = {0xB5, 0x62, 0x06, 0x31, 0x20, 0x00, 0x00, 0x01, 0x00, 0x00, 0x32, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x96, 0x98, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00,
                   0x00, 0x00, 0x6F, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setTP5[2], sizeof(setTP5) - 4);

  // Set SBAS mode
  byte setSBAS[] = {0xB5, 0x62, 0x06, 0x16, 0x08, settingsArrayPointer[12], 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  calcChecksum(&setSBAS[2], sizeof(setSBAS) - 4);

  // Start with setting the NAV mode
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3)
  {
    sendUBX(&setNav[0], sizeof(setNav));  //Send UBX Packet
    gpsSetSuccess += getUBX_ACK(&setNav[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[0] = '+';
  } else {
    GPAsuccess[0] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.setCursor(9, 2);
    lcd.print(GPAsuccess[0]);
  #endif
  
  // Set the update frequency
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(&setDataRate[0], sizeof(setDataRate));  //Send UBX Packet
    gpsSetSuccess += getUBX_ACK(&setDataRate[2]); //Passes Class ID and Message ID to the ACK Receive function
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[1] = '+';
  } else {
    GPAsuccess[1] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004 or LCD_1602
    lcd.print(GPAsuccess[1]);
  #endif
  
  // Set the GLL messages
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setGLL, sizeof(setGLL));
    gpsSetSuccess += getUBX_ACK(&setGLL[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[2] = '+';
  } else {
    GPAsuccess[2] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[2]);
  #endif
  // Set the GSA messages
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setGSA, sizeof(setGSA));
    gpsSetSuccess += getUBX_ACK(&setGSA[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[3] = '+';
  } else {
    GPAsuccess[3] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[3]);
  #endif
  // Set the GSV messages  
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setGSV, sizeof(setGSV));
    gpsSetSuccess += getUBX_ACK(&setGSV[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[4] = '+';
  } else {
    GPAsuccess[4] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[4]);
  #endif
  // Set the RMC messages  
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setRMC, sizeof(setRMC));
    gpsSetSuccess += getUBX_ACK(&setRMC[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[5] = '+';
  } else {
    GPAsuccess[5] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[5]);
  #endif
  // Set the VTG messages
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setVTG, sizeof(setVTG));
    gpsSetSuccess += getUBX_ACK(&setVTG[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[6] = '+';
  } else {
    GPAsuccess[6] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[6]);
  #endif
  // Set power mode
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setPWRMG, sizeof(setPWRMG));
    gpsSetSuccess += getUBX_ACK(&setPWRMG[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[7] = '+';
  } else {
    GPAsuccess[7] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[7]);
  #endif
   // Set the the timepulse. 1Hz and no fix, 10MHz when fixed
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setTP5, sizeof(setTP5));
    gpsSetSuccess += getUBX_ACK(&setTP5[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[8] = '+';
  } else {
    GPAsuccess[8] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[8]);
  #endif
  // Set the SBAS system
  UbloxSmartWait (SafeWait);
  gpsSetSuccess = 0;
  while(gpsSetSuccess < 3) {
    sendUBX(setSBAS, sizeof(setSBAS));
    gpsSetSuccess += getUBX_ACK(&setSBAS[2]);
    if (gpsSetSuccess == 5) gpsSetSuccess -= 4;
  }
  if (gpsSetSuccess == 10) {
    GPAsuccess[9] = '+';
  } else {
    GPAsuccess[9] = '-';
    GpsStatus = false;
  }
  #ifdef LCD_2004
    lcd.print(GPAsuccess[9]);
  #endif

  if (settingsArrayPointer[4] != 0x25) {
    //sendUBX(&setPortRate[0], sizeof(setPortRate));
    ///setBaud(settingsArrayPointer[4]);
    //UbloxSmartWait (SafeWait);      
    //  if (USB_DEBUG) Serial.println("GPS init success and baudrate raised");
  }
}

  //eof
